(function () {
    'use strict';

    (function () {
        if (typeof window.CustomEvent === "function")
            return false;
        function CustomEvent(event, params) {
            params = params || { bubbles: false, cancelable: false, detail: undefined };
            var evt = document.createEvent("CustomEvent");
            evt.initCustomEvent(event, params.bubbles, params.cancelable, params.detail);
            return evt;
        }
        CustomEvent.prototype = window.Event.prototype;
        //@ts-ignore
        window["CustomEvent"] = CustomEvent;
    })();
    var CF7RepeaterCollectionField = /** @class */ (function () {
        function CF7RepeaterCollectionField() {
            var _this = this;
            this.fields = [];
            this.ready(function () {
                if (typeof wpcf_repeater == "undefined" || wpcf_repeater.length == 0) {
                    return;
                }
                for (var i = 0; i < wpcf_repeater.length; i++) {
                    _this.fields.push(new CF7RepeaterField(wpcf_repeater[i]));
                }
            });
        }
        CF7RepeaterCollectionField.prototype.ready = function (fn) {
            if (document.readyState != "loading") {
                fn();
            }
            else {
                document.addEventListener("DOMContentLoaded", fn);
            }
        };
        return CF7RepeaterCollectionField;
    }());
    var CF7RepeaterField = /** @class */ (function () {
        function CF7RepeaterField(field) {
            var _this = this;
            this.count = 0;
            this.initialDisplay = "";
            this.field = field;
            if (!this.field) {
                return;
            }
            this.element = document.querySelector(".wpcf7-repeater-" + this.field.id);
            this.template = this.element.querySelector(".wpcf7-repeater-content").innerHTML;
            this.listElement = this.element.querySelector(".wpcf7-repeater-list");
            this.addElement = this.element.querySelector("a.wpcf7-repeater-add");
            this.initialDisplay = this.addElement.style.display;
            if (this.field.min > 0) {
                for (var j = 0; j < this.field.min; j++) {
                    this.create();
                    var removeElement = this.element.querySelector("a.wpcf7-repeater-remove");
                    if (removeElement) {
                        removeElement.parentNode.removeChild(removeElement);
                    }
                }
                this.count = this.field.min - 1;
            }
            if (this.field.show > this.count) {
                var to = this.field.show - this.count;
                for (var j = 0; j < to; j++) {
                    this.create();
                }
                this.count = this.field.show;
            }
            this.addElement.addEventListener("click", function (event) {
                event.preventDefault();
                _this.create();
                return false;
            });
        }
        CF7RepeaterField.prototype.create = function () {
            var _this = this;
            if (this.count >= this.field.max && this.field.max >= 0) {
                return;
            }
            var html = this.template.replace(/{{repeater}}/g, this.field.index).replace(/{{index}}/g, this.field.item);
            this.count = this.count + 1;
            this.field.item = this.field.item + 1;
            var element = document.createElement("div");
            element.classList.add("wpcf7-repeater-item");
            element.innerHTML = html;
            this.listElement.appendChild(element);
            if (this.count >= this.field.max - 1 && this.field.max >= 0) {
                this.addElement.style.display = "none";
            }
            element.querySelector("a.wpcf7-repeater-remove").addEventListener("click", function (event) {
                event.preventDefault();
                _this.listElement.removeChild(element);
                _this.count = _this.count - 1;
                if (_this.count < _this.field.max - 1 && _this.field.max >= 0) {
                    _this.addElement.style.display = _this.initialDisplay;
                }
                _this.trigger("wpcf7-panel-created", [_this.field, _this]);
                return false;
            });
            this.trigger("wpcf7-panel-created", [this.field, html, element, this]);
            if (wpcf7.jqueryUi && !wpcf7.supportHtml5.date) {
                [].forEach.call(element.querySelectorAll('input.wpcf7-date[type="date"]'), function (input) {
                    if (typeof jQuery != "undefined") {
                        jQuery(input).datepicker({
                            dateFormat: "yy-mm-dd",
                            minDate: new Date(input.getAttribute("min")),
                            maxDate: new Date(input.getAttribute("max"))
                        });
                    }
                });
            }
            if (wpcf7.jqueryUi && !wpcf7.supportHtml5.number) {
                [].forEach.call(element.querySelectorAll('input.wpcf7-number[type="number"]'), function (input) {
                    if (typeof jQuery != "undefined") {
                        jQuery(input).spinner({
                            min: input.getAttribute("min"),
                            max: input.getAttribute("max"),
                            step: input.getAttribute("step")
                        });
                    }
                });
            }
        };
        CF7RepeaterField.prototype.trigger = function (name, detail) {
            // Create a new event
            var event = new CustomEvent(name, { detail: detail });
            // Dispatch the event
            document.dispatchEvent(event);
        };
        return CF7RepeaterField;
    }());
    window['CF7RepeaterCollectionField'] = new CF7RepeaterCollectionField();

}());
