<?php

namespace Ajaxy\Cf7Field\Repeater;

class Validator
{
    public function __construct($schema)
    {
        $this->schema = $schema;
    }
    public function add($tag)
    {
        switch ($tag->basetype) {
            case 'file':
                if ($tag->is_required()) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('requiredfile', array(
                            'field' => $tag->name,
                            'error' => \wpcf7_get_message('invalid_required'),
                        ))
                    );
                }

                $this->schema->add_rule(
                    \wpcf7_swv_create_rule('file', array(
                        'field' => $tag->name,
                        'accept' => explode(',', \wpcf7_acceptable_filetypes(
                            $tag->get_option('filetypes'),
                            'attr'
                        )),
                        'error' => \wpcf7_get_message('upload_file_type_invalid'),
                    ))
                );

                $this->schema->add_rule(
                    \wpcf7_swv_create_rule('maxfilesize', array(
                        'field' => $tag->name,
                        'threshold' => $tag->get_limit_option(),
                        'error' => \wpcf7_get_message('upload_file_too_large'),
                    ))
                );
                break;
            case 'select';
                if ($tag->is_required()) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('required', array(
                            'field' => $tag,
                            'error' => \wpcf7_get_message('invalid_required'),
                        ))
                    );
                }
                break;
            case 'number':
                if ($tag->is_required()) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('required', array(
                            'field' => $tag->name,
                            'error' => \wpcf7_get_message('invalid_required'),
                        ))
                    );
                }

                $this->schema->add_rule(
                    \wpcf7_swv_create_rule('number', array(
                        'field' => $tag->name,
                        'error' => \wpcf7_get_message('invalid_number'),
                    ))
                );

                $min = $tag->get_option('min', 'signed_num', true);
                $max = $tag->get_option('max', 'signed_num', true);

                if ('range' === $tag->basetype) {
                    if (!\wpcf7_is_number($min)) {
                        $min = '0';
                    }

                    if (!\wpcf7_is_number($max)) {
                        $max = '100';
                    }
                }

                if (\wpcf7_is_number($min)) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('minnumber', array(
                            'field' => $tag->name,
                            'threshold' => $min,
                            'error' => \wpcf7_get_message('number_too_small'),
                        ))
                    );
                }

                if (\wpcf7_is_number($max)) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('maxnumber', array(
                            'field' => $tag->name,
                            'threshold' => $max,
                            'error' => \wpcf7_get_message('number_too_large'),
                        ))
                    );
                }
                break;
            case 'date':
                if ($tag->is_required()) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('required', array(
                            'field' => $tag->name,
                            'error' => \wpcf7_get_message('invalid_required'),
                        ))
                    );
                }

                $this->schema->add_rule(
                    \wpcf7_swv_create_rule('date', array(
                        'field' => $tag->name,
                        'error' => \wpcf7_get_message('invalid_date'),
                    ))
                );

                $min = $tag->get_date_option('min');
                $max = $tag->get_date_option('max');

                if (false !== $min) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('mindate', array(
                            'field' => $tag->name,
                            'threshold' => $min,
                            'error' => \wpcf7_get_message('date_too_early'),
                        ))
                    );
                }

                if (false !== $max) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('maxdate', array(
                            'field' => $tag->name,
                            'threshold' => $max,
                            'error' => \wpcf7_get_message('date_too_late'),
                        ))
                    );
                }
                break;
            default:
                if ($tag->is_required()) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('required', array(
                            'field' => $tag,
                            'error' => \wpcf7_get_message('invalid_required'),
                        ))
                    );
                }

                if ('email' === $tag->basetype) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('email', array(
                            'field' => $tag,
                            'error' => \wpcf7_get_message('invalid_email'),
                        ))
                    );
                }

                if ('url' === $tag->basetype) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('url', array(
                            'field' => $tag,
                            'error' => \wpcf7_get_message('invalid_url'),
                        ))
                    );
                }

                if ('tel' === $tag->basetype) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('tel', array(
                            'field' => $tag,
                            'error' => \wpcf7_get_message('invalid_tel'),
                        ))
                    );
                }

                if ($minlength = $tag->get_minlength_option()) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('minlength', array(
                            'field' => $tag,
                            'threshold' => \absint($minlength),
                            'error' => \wpcf7_get_message('invalid_too_short'),
                        ))
                    );
                }

                if ($maxlength = $tag->get_maxlength_option()) {
                    $this->schema->add_rule(
                        \wpcf7_swv_create_rule('maxlength', array(
                            'field' => $tag,
                            'threshold' => \absint($maxlength),
                            'error' => \wpcf7_get_message('invalid_too_long'),
                        ))
                    );
                }
                break;
        }
    }
}
